unit UDrawLine;

interface

uses
   Classes,Controls,SysUtils;

type
   TDrawLine=class;
   TLines=class;
   TLine=class;
   TAccessCustomControl=class(TCustomControl)
   end;
   TLineClass=class of TLine;

   TDrawLine=class(TComponent)
   private
     FLines:TLines;
     FLinkControl:TCustomControl;
     procedure SetLines(Value: TLines);
     function  CreateLine:TLine;
     function  CreateLines:TLines;
   protected
     function GetLineClass: TLineClass; virtual;
   public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     procedure DrawLines;
   published
     property Lines:TLines read FLines write SetLines;
     property LinkControl:TCustomControl read FLinkControl write FLinkControl;
   end;

   TLines = class(TCollection)
    private
      FDrawLine: TDrawLine;
      function GetItem(Index: Integer): TLine;
      procedure SetItem(Index: Integer; Value: TLine);
    protected
      function GetOwner: TPersistent; override;
    public
      constructor Create(ADrawLine: TDrawLine);
      function Add: TLine;
      function AddItem(Item: TLine; Index: Integer): TLine;
      function Insert(Index: Integer): TLine;
      property Items[Index: Integer]: TLine read GetItem write SetItem; default;
    end;

   TLine = class(TCollectionItem)
   private
     FFromX:Integer;
     FFromY:Integer;
     FToX:Integer;
     FToY:Integer;
   protected

   public
     constructor Create(Collection: TCollection); override;  
   published
     property FromX:Integer read FFromX write FFromX;
     property FromY:Integer read FFromY write FFromY;
     property ToX:Integer read FToX write FToX;
     property ToY:Integer read FToY write FToY;

   end;

procedure Register;

implementation

{ TLine }

constructor TLine.Create(Collection: TCollection);
begin
  inherited Create(Collection);

end;

{ TLines }

function TLines.Add: TLine;
begin
  Result := TLine(inherited Add);
end;

function TLines.AddItem(Item: TLine; Index: Integer): TLine;
begin
  if Item = nil then
    Result := FDrawLine.CreateLine
  else
    Result := Item;
  if Assigned(Result) then
  begin
    Result.Collection := Self;
    if Index < 0 then
      Index := Count - 1;
    Result.Index := Index;
  end;
end;

constructor TLines.Create(ADrawLine: TDrawLine);
begin
  if FDrawLine <> nil then
    inherited Create(FDrawLine.GetLineClass)
  else
    inherited Create(TLine);
  FDrawLine := ADrawLine;
end;

function TLines.GetItem(Index: Integer): TLine;
begin
  Result := TLine(inherited GetItem(Index));
end;

function TLines.GetOwner: TPersistent;
begin
  Result:=FDrawLine;
end;

function TLines.Insert(Index: Integer): TLine;
begin
  Result := AddItem(nil, Index);
end;

procedure TLines.SetItem(Index: Integer; Value: TLine);
begin
  inherited SetItem(Index, Value);
end;

{ TDrawLine }

constructor TDrawLine.Create(AOwner: TComponent);
begin
  inherited;
  FLines := CreateLines;
end;

function TDrawLine.CreateLine: TLine;
begin
  Result:=TLine.Create(Lines);
end;

function TDrawLine.CreateLines:TLines;
begin
  Result:=TLines.Create(Self);
end;

destructor TDrawLine.Destroy;
begin
  FreeAndNil(FLines);
  inherited;
end;

procedure TDrawLine.DrawLines;
var
  I:Integer;
  tmpobj_Line:TLine;
begin
  if Assigned(FLinkControl) then
  begin
    for I:=0 to FLines.Count-1 do
    begin
      tmpobj_Line:=FLines.Items[I];
      with TAccessCustomControl(FLinkControl) do
      begin
        Canvas.MoveTo(tmpobj_Line.FFromX,tmpobj_Line.FFromY);
        Canvas.LineTo(tmpobj_Line.FToX,tmpobj_Line.FToY);
      end;
    end;
  end;
end;

function TDrawLine.GetLineClass: TLineClass;
begin
  Result := TLine;
end;

procedure TDrawLine.SetLines(Value: TLines);
begin
  FLines.Assign(Value);
end;

procedure Register;
begin
  RegisterComponents('Samples', [TDrawLine]);
end;

end.
 